/* GtkJentchina
 * STRASSER Michel - 20040313		init
 * STRASSER Michel - 20040410		Now herits from GtkDialog (GnomeDialog is deprecated)
 * */


#include "GJnaFilterSelectionDialog.h"
#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include "GJnaFsdInfo.h"
#include "GJnaFsdSelect.h"
#include "GJnaFsdIFAAuthor.h"
#include "../config.h" // for H_NumPics

static GObjectClass *gjnaFilterSelectionDialog_parentClass;

static void
impl_GJnaFilterSelectionDialog_finalize(GObject *object)
{
	GJnaFilterSelectionDialog *self=(GJnaFilterSelectionDialog*)object;

	gtk_widget_hide(GTK_WIDGET(self));
	g_object_unref(G_OBJECT(self->hPaned));
	g_object_unref(G_OBJECT(self->select));
	g_object_unref(G_OBJECT(self->info));
	g_object_unref(G_OBJECT(self->ok));
	g_object_unref(G_OBJECT(self->cancel));
	gjnaFilterSelectionDialog_parentClass->finalize(object);
}

static void
GJnaFilterSelectionDialogClass_init (GJnaFilterSelectionDialogClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	gjnaFilterSelectionDialog_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_GJnaFilterSelectionDialog_finalize;
}

GType		GJnaFilterSelectionDialog_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (GJnaFilterSelectionDialogClass),
			NULL, NULL,
			(GClassInitFunc) GJnaFilterSelectionDialogClass_init,
			NULL, NULL,
			sizeof (GJnaFilterSelectionDialog),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			GTK_TYPE_DIALOG,
			"GJnaFilterSelectionDialog", &info, 0);
	}

	return type;
}

GObject* 	
GJnaFilterSelectionDialog_new		(char* appName,
					 GJnaFilterType filterType,
					 GObject* jentchina)
{
	GJnaFilterSelectionDialog* retval;
	GtkWidget* vbox;
	GtkWidget *dialog_action_area;
	char* title;

	g_message(__PRETTY_FUNCTION__);
	
	retval=g_object_new(GJNA_TYPE_FILTER_SELECTION_DIALOG, NULL);
	retval->jentchina=jentchina;
	gtk_window_set_resizable(GTK_WINDOW(retval), TRUE);
	title=g_strconcat((filterType==GJnaFilterType_IMPORT)?"Importation ":"Exportation ", appName, NULL);
	gtk_window_set_title(GTK_WINDOW(retval), title);
	g_free(title);

	//gtk_window_set_policy(GTK_WINDOW (retval), FALSE, FALSE, FALSE);
	vbox = GTK_DIALOG(retval)->vbox;
	gtk_widget_show(vbox);
		
	dialog_action_area = GTK_DIALOG (retval)->action_area;
	gtk_widget_show (dialog_action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area), GTK_BUTTONBOX_END);

	retval->cancel = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (retval->cancel);
	gtk_dialog_add_action_widget (GTK_DIALOG (retval), retval->cancel, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS (retval->cancel, GTK_CAN_DEFAULT);

	retval->ok = gtk_button_new_from_stock ("gtk-ok");
	gtk_widget_show(retval->ok);
	gtk_dialog_add_action_widget (GTK_DIALOG (retval), retval->ok, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS (retval->ok, GTK_CAN_DEFAULT);

	retval->hPaned=gtk_hpaned_new();
	g_object_ref(G_OBJECT(retval->hPaned));
	gtk_widget_show(GTK_WIDGET(retval->hPaned));

	gtk_box_pack_start (GTK_BOX(vbox), retval->hPaned, TRUE, TRUE, 0);
	
	retval->select=(GtkWidget*)GJnaFsdSelect_new(jentchina, filterType, G_OBJECT(retval));
	gtk_widget_show(GTK_WIDGET(retval->select));
	
	gtk_paned_add1(GTK_PANED(retval->hPaned), retval->select);

	retval->info=(GtkWidget*)GJnaFsdInfo_new();
	g_object_ref(G_OBJECT(retval->info));
	gtk_widget_show(GTK_WIDGET(retval->info));
	
	gtk_paned_add2(GTK_PANED(retval->hPaned), retval->info);
	
	return (GObject*) retval;
}

void
GJnaFilterSelectionDialog_CB_selectionChanged	(GJnaFilterSelectionDialog* self,
						 JentchinaIter iter)
{
	char* serverName;		/* newly allocated */ 
	char* serverPic;		/* newly allocated */
	char* filterName;		/* newly allocated */
	int version;
	char* year;			/* newly allocated */
	char* license;			/* newly allocated */
	char* application;		/* newly allocated */
	char* info;			/* newly allocated */
	GList* authors;			/* newly allocated, list of string */
	char* filterPic;		/* newly allocated */
	JentchinaStatus status;
	GList* l;
	GList* ifAuthors;

	g_print("%s, Selected :", __PRETTY_FUNCTION__);
	if(self->jentchina) {
		status=JentchinaObj_getInfos(JENTCHINA_OBJ(self->jentchina), iter, &serverName, &serverPic, &filterName, &version, 
				&year, &license, &application, &info, &authors, &filterPic);

		self->filterRef=iter;
		if(filterName)
			gtk_widget_set_sensitive(GTK_WIDGET(self->ok), TRUE);
		else
			gtk_widget_set_sensitive(GTK_WIDGET(self->ok), FALSE);

		g_print("serverName:'%s', serverPic:'%s', filterName:'%s', version:'%d', year:'%s', license:'%s', " 
				"application:'%s', info:'%s', authors:[", serverName, serverPic, filterName, version,
				year, license, application, info);

		ifAuthors=NULL;
		for(l=authors; l ; l=g_list_next(l)) {
			char* id=(char*)l->data;
			JentchinaStatus status2;
			char *name=NULL, *email=NULL, *picture=NULL;
			GObject* ifAAuthor;
			
			g_print("'%s', ", (char*)l->data);
			status2=JentchinaObj_getAuthorInfos(JENTCHINA_OBJ(self->jentchina), iter, 
					id, &name, &email, &picture);
			if(!picture) {
				char rad[]="0";

				rad[0]=((id[0] % H_NumPics) % 10) + '0';
				picture=g_strconcat(PICS_DIR, "AuthorPic-", rad, ".png", NULL);
			}
			if(!name)
				name=g_strconcat("<i>", id, "</i>", NULL);
			if(!email)
				email=g_strdup("NoEmail");
			ifAAuthor=GJnaFsdIFAAuthor_new(name, email, picture);
			ifAuthors=g_list_append(ifAuthors, ifAAuthor);
			if(name)
				g_free(name);
			if(email)
				g_free(email);
			if(picture)
				g_free(picture);
		}
		g_print("], filterPic:'%s'\n", filterPic);
	
		GJnaFsdInfo_set(GJNA_FSD_INFO(self->info), serverName, serverPic, filterName, version,
					year, license, application, info, ifAuthors, filterPic);

		for(l=ifAuthors; l ; l=g_list_next(l))
			g_object_unref(l->data);

		
		if(serverName)
			g_free(serverName);
		if(serverPic)
			g_free(serverPic);
		if(filterName)
			g_free(filterName);
		if(year)
			g_free(year);
		if(license)
			g_free(license);
		if(application)
			g_free(application);
		if(info)
			g_free(info);
		if(filterPic)
			g_free(filterPic);
		
		for(l=authors; l ; l=g_list_next(l))
			g_free(l->data);

		g_list_free(l);
	}
}


