/* GtkJentchina
 * STRASSER Michel - 20040411		init
 * */

#include "GJnaFsdModel.h"
#include <glib.h>
#include <glib-object.h>
#include <libJentchina.h>

static GObjectClass *gjnaFsdModel_parentClass;

// #define GJnaFsdModel_DEBUG

#ifdef GJnaFsdModel_DEBUG 
#define GJnaFsdModel_COMMON_DEBUG_FUNC_2() \
	g_message("%s, line %d", __PRETTY_FUNCTION__, __LINE__); 
#else
#define GJnaFsdModel_COMMON_DEBUG_FUNC_2() ;
#endif


#ifdef GJnaFsdModel_DEBUG 
#define GJnaFsdModel_COMMON_DEBUG_FUNC() \
	if(iter) \
		g_message("%s, line %d, iter=%d, iter->user_data=%u", __PRETTY_FUNCTION__, __LINE__, (int)iter, (int)iter->user_data); \
	else \
		g_message("%s, line %d, iter is NULL", __PRETTY_FUNCTION__, __LINE__);
#else
#define GJnaFsdModel_COMMON_DEBUG_FUNC() ;
#endif

static void
impl_GJnaFsdModel_msg					(GJnaFsdModel* self,
							 char* func,
							 int line,
							 GtkTreeIter* iter)
{
#ifdef GJnaFsdModel_DEBUG 
	if(iter)  {
		char *str=NULL; \
		str=JentchinaTreeModel_getPath(JENTCHINA_TREE_MODEL(self->model), GPOINTER_TO_INT(iter->user_data));
		
		g_message("%s, line %d, iter=%d, iter->user_data=%u, path=%s", func, line, (int)iter, (int)iter->user_data, str); 
		if(str) 
			g_free(str); 
	
	} 
	else 
		g_message("%s, line %d, iter is NULL", __PRETTY_FUNCTION__, __LINE__); 
#endif
}

/*  whether our tree model
 *  has any special characteristics. Each
 *  tree iter is valid as long as the row in question
 *  exists.
 */

static GtkTreeModelFlags
impl_GJnaFsdModel_GtkTreeModelInterface_getFlags 	(GtkTreeModel *treeModel)
{ 
	GJnaFsdModel_COMMON_DEBUG_FUNC_2();
	g_return_val_if_fail (GJNA_IS_FSD_MODEL(treeModel), (GtkTreeModelFlags)0);

	return (GTK_TREE_MODEL_ITERS_PERSIST);
}


/* how many data columns we export via the tree model interface */

static gint
impl_GJnaFsdModel_GtkTreeModelInterface_getNColumns 	(GtkTreeModel *treeModel)
{
	gint retval;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	
	GJnaFsdModel_COMMON_DEBUG_FUNC_2();
	g_return_val_if_fail (GJNA_IS_FSD_MODEL(treeModel), 0);
	retval=JentchinaTreeModel_getNColumns(JENTCHINA_TREE_MODEL(self->model));

	/* g_message("NColumns=%d", retval); */
	return retval;
}

/* which type of data an exported model column contains */

static GType
impl_GJnaFsdModel_GtkTreeModelInterface_getColumnType 		(GtkTreeModel *treeModel,
                             					 gint index)
{
	gint retval;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	
	GJnaFsdModel_COMMON_DEBUG_FUNC_2();
	g_return_val_if_fail (GJNA_IS_FSD_MODEL(treeModel), G_TYPE_INVALID);
	retval=JentchinaTreeModel_getColumnType(JENTCHINA_TREE_MODEL(self->model), index);

	/* g_message("ColumnType[column=%d]=%d", index, retval); */
	return retval;
}

/* converts a tree path (physical position) into a tree iter structure 
 * (the content of the iter fields will only be used internally by our model).
 */

static gboolean
impl_GJnaFsdModel_GtkTreeModelInterface_getIter 		(GtkTreeModel *treeModel,
		                      				 GtkTreeIter  *iter,
								 GtkTreePath  *path)
{
	gboolean retval;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	char* pathS=NULL;
	JentchinaIter jIter;
	
	//GJnaFsdModel_COMMON_DEBUG_FUNC();


	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), FALSE);
	g_return_val_if_fail(path, FALSE);
  	g_return_val_if_fail (iter, FALSE);
	
	pathS=gtk_tree_path_to_string(path);
	/* g_message("pathS is %s", pathS); */
	g_return_val_if_fail(pathS, FALSE);
	retval=JentchinaTreeModel_getIter(JENTCHINA_TREE_MODEL(self->model), &jIter, pathS);
	g_return_val_if_fail(jIter, FALSE);
	iter->user_data=GINT_TO_POINTER(jIter);

	//GJnaFsdModel_COMMON_DEBUG_FUNC();
	impl_GJnaFsdModel_msg(self, __PRETTY_FUNCTION__, __LINE__, iter);
	g_free(pathS);
	
	return retval;
}


/* converts a tree iter into a tree path (ie. the physical position of that row in the list). */

static GtkTreePath *
impl_GJnaFsdModel_GtkTreeModelInterface_getPath 		(GtkTreeModel *treeModel,
								 GtkTreeIter  *iter)
{	
	GtkTreePath* retval;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter;
	char* pathS;
	
	GJnaFsdModel_COMMON_DEBUG_FUNC();

	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), NULL);
  	g_return_val_if_fail (iter, NULL);

	jIter=GPOINTER_TO_INT(iter->user_data);
	
	pathS=JentchinaTreeModel_getPath(JENTCHINA_TREE_MODEL(self->model), jIter);
	g_return_val_if_fail(pathS, NULL);
	retval=gtk_tree_path_new_from_string(pathS);
	g_free(pathS);
	g_return_val_if_fail(retval, NULL);
	
	return retval;
}

/* Returns a row's exported data columns (_get_value is what gtk_treeModel_get uses) */

static void
impl_GJnaFsdModel_GtkTreeModelInterface_getValue 		(GtkTreeModel *treeModel,
								 GtkTreeIter  *iter,
								 gint column,
								 GValue       *value)
{
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter;
		
	GJnaFsdModel_COMMON_DEBUG_FUNC();


	g_return_if_fail(GJNA_IS_FSD_MODEL(treeModel));
  	g_return_if_fail(iter);
  	g_return_if_fail(value);

	jIter=GPOINTER_TO_INT(iter->user_data);
	JentchinaTreeModel_getValue(JENTCHINA_TREE_MODEL(self->model), jIter, column, value);
}


/* Takes an iter structure and sets it to point to the next row. */

static gboolean
impl_GJnaFsdModel_GtkTreeModelInterface_iterNext 		(GtkTreeModel* treeModel,
								 GtkTreeIter   *iter)
{
	gboolean retval=TRUE;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter;
		
	//GJnaFsdModel_COMMON_DEBUG_FUNC();

	impl_GJnaFsdModel_msg(self, __PRETTY_FUNCTION__, __LINE__, iter);

	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), FALSE);
  	g_return_val_if_fail(iter, FALSE);

	jIter=GPOINTER_TO_INT(iter->user_data);
	retval=JentchinaTreeModel_iterNext(JENTCHINA_TREE_MODEL(self->model), &jIter);

	iter->user_data=GINT_TO_POINTER(jIter);

	return retval;
}


/* Returns TRUE or FALSE depending on whether
 * the row specified by 'parent' has any children.
 * If it has children, then 'iter' is set to
 * point to the first child. Special case: if
 * 'parent' is NULL, then the first top-level
 * row should be returned if it exists.
 */

static gboolean
impl_GJnaFsdModel_GtkTreeModelInterface_iterChildren 			(GtkTreeModel *treeModel,
									 GtkTreeIter  *iter,
									 GtkTreeIter  *parent)
{
	gboolean retval=TRUE;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter, jIParent;
		
	GJnaFsdModel_COMMON_DEBUG_FUNC();


	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), FALSE);
  	g_return_val_if_fail(iter, FALSE);

	if(parent) 
		jIParent=GPOINTER_TO_INT(parent->user_data);
	else
		jIParent=0;
	
	retval=JentchinaTreeModel_iterChildren(JENTCHINA_TREE_MODEL(self->model), &jIter, jIParent);
		
	iter->user_data=GINT_TO_POINTER(jIter);

	return retval;
}

/* Returns TRUE or FALSE depending on whether the row specified by 'iter' has any children. */

static gboolean
impl_GJnaFsdModel_GtkTreeModelInterface_iterHasChild 			(GtkTreeModel *treeModel,
									 GtkTreeIter  *iter)
{
	gboolean retval=TRUE;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter;
		
	GJnaFsdModel_COMMON_DEBUG_FUNC();


	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), FALSE);

	if(iter)
		jIter=GPOINTER_TO_INT(iter->user_data);
	else
		jIter=0;
	
	retval=JentchinaTreeModel_iterHasChild(JENTCHINA_TREE_MODEL(self->model), jIter);

	return retval;
}

/* Returns the number of children the row
 * specified by 'iter' has. This is usually 0,
 * as we only have a list and thus do not have
 * any children to any rows. A special case is
 * when 'iter' is NULL, in which case we need
 * to return the number of top-level nodes,
 * ie. the number of rows in our list.
 */

static gint
impl_GJnaFsdModel_GtkTreeModelInterface_iterNChildren 			(GtkTreeModel* treeModel,
									 GtkTreeIter  *iter)
{
	gint retval=0;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter;
		
	GJnaFsdModel_COMMON_DEBUG_FUNC();


	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), 0);

	if(iter)
		jIter=GPOINTER_TO_INT(iter->user_data);
	else
		jIter=0;
	
	retval=JentchinaTreeModel_iterNChildren(JENTCHINA_TREE_MODEL(self->model), jIter);

	return retval;
}

/* If the row specified by 'parent' has any
 * children, set 'iter' to the n-th child and
 * return TRUE if it exists, otherwise FALSE.
 * A special case is when 'parent' is NULL, in
 * which case we need to set 'iter' to the n-th 
 * row if it exists.
 */

static gboolean
impl_GJnaFsdModel_GtkTreeModelInterface_iterNthChild			(GtkTreeModel *treeModel,
									 GtkTreeIter  *iter,
									 GtkTreeIter  *parent,
									 gint n)
{
	gboolean retval=TRUE;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter, jIParent;
		
	/* g_message("n=%d", n); */
	/*
	if(parent) \
		g_message("%s, line %d, parent=%d, parent->user_data=%d, n=%d", __PRETTY_FUNCTION__, __LINE__, (int)parent, (int)parent->user_data, n); \
	else \
		g_message("%s, line %d, parent is NULL, n=%d", __PRETTY_FUNCTION__, __LINE__, n); 
	*/

	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), FALSE);
  	g_return_val_if_fail(iter, FALSE);

	if(parent)
		jIParent=GPOINTER_TO_INT(parent->user_data);
	else
		jIParent=0;
	
	retval=JentchinaTreeModel_iterNthChild(JENTCHINA_TREE_MODEL(self->model), &jIter, jIParent, n);

	iter->user_data=GINT_TO_POINTER(jIter);

	GJnaFsdModel_COMMON_DEBUG_FUNC();

	return retval;

}

/* Point 'iter' to the parent node of 'child'. As
 * we have a list and thus no children and no 
 * parents of children, we can just return FALSE.
 */

static gboolean
impl_GJnaFsdModel_GtkTreeModelInterface_iterParent 		(GtkTreeModel *treeModel,
								 GtkTreeIter  *iter,
								 GtkTreeIter* child)
{
	gboolean retval=TRUE;
	GJnaFsdModel* self=(GJnaFsdModel*)treeModel;
	JentchinaIter jIter, jIChild;
		
	GJnaFsdModel_COMMON_DEBUG_FUNC();


	g_return_val_if_fail(GJNA_IS_FSD_MODEL(treeModel), FALSE);
  	g_return_val_if_fail(iter, FALSE);
	g_return_val_if_fail (child == NULL || child->user_data != NULL, FALSE);

	jIChild=GPOINTER_TO_INT(child->user_data);
	retval=JentchinaTreeModel_iterParent(JENTCHINA_TREE_MODEL(self->model), &jIter, jIChild);

	iter->user_data=GINT_TO_POINTER(jIter);

	return retval;
}

static void
impl_GJnaFsdModel_finalize(GObject *object)
{
	GJnaFsdModel* self=(GJnaFsdModel*)object;
	
	g_object_unref(G_OBJECT(self->model));

	gjnaFsdModel_parentClass->finalize(object);
}

static void
GJnaFsdModelClass_init (GJnaFsdModelClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	gjnaFsdModel_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_GJnaFsdModel_finalize;
}

static void
impl_GJnaFsdModel_GtkTreeModelInterface_init	(GtkTreeModelIface *iface)
{
	iface->get_flags       = impl_GJnaFsdModel_GtkTreeModelInterface_getFlags;
	iface->get_n_columns   = impl_GJnaFsdModel_GtkTreeModelInterface_getNColumns;
	iface->get_column_type = impl_GJnaFsdModel_GtkTreeModelInterface_getColumnType;
	iface->get_iter        = impl_GJnaFsdModel_GtkTreeModelInterface_getIter;
	iface->get_path        = impl_GJnaFsdModel_GtkTreeModelInterface_getPath;
	iface->get_value       = impl_GJnaFsdModel_GtkTreeModelInterface_getValue;
	iface->iter_next       = impl_GJnaFsdModel_GtkTreeModelInterface_iterNext;
	iface->iter_children   = impl_GJnaFsdModel_GtkTreeModelInterface_iterChildren;
	iface->iter_has_child  = impl_GJnaFsdModel_GtkTreeModelInterface_iterHasChild;
	iface->iter_n_children = impl_GJnaFsdModel_GtkTreeModelInterface_iterNChildren;
	iface->iter_nth_child  = impl_GJnaFsdModel_GtkTreeModelInterface_iterNthChild;
	iface->iter_parent     = impl_GJnaFsdModel_GtkTreeModelInterface_iterParent;
}

GType		GJnaFsdModel_getType		(void)
{
	static GType type = 0;

	if (!type) {
		const GTypeInfo info = {
			sizeof (GJnaFsdModelClass),
			NULL, NULL,
			(GClassInitFunc) GJnaFsdModelClass_init,
			NULL, NULL,
			sizeof (GJnaFsdModel),
			0,
			NULL, NULL
		};
		static const GInterfaceInfo iInfo0 ={(GInterfaceInitFunc) impl_GJnaFsdModel_GtkTreeModelInterface_init, NULL, NULL};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"GJnaFsdModel", &info, 0);

    		g_type_add_interface_static (type, GTK_TYPE_TREE_MODEL, &iInfo0);
	}

	return type;
}

GObject* 	
GJnaFsdModel_new			(GObject* model)
{
	GJnaFsdModel* retval;

	g_return_val_if_fail(JENTCHINA_TREE_MODEL(model), NULL);
	
	retval=g_object_new(GJNA_TYPE_FSD_MODEL, NULL);
	retval->model=model;
	g_object_ref(model);
	
	return (GObject*) retval;
}

