/* GtkJentchina
 * STRASSER Michel - 20040317		init (GJnaFsdSServerView)
 * STRASSER Michel - 20040413		renamed to GJnaFsdSView, model type is passed as a parameter in the constructor
 * */


#include "GJnaFsdSView.h"
#include <glib.h>
#include <glib-object.h>
#include <gnome.h>
#include <libJentchina.h>
#include "GJnaFsdModel.h"
#include "GJnaFilterSelectionDialog.h"

static GObjectClass *gjnaFsdSView_parentClass;

static void
impl_GJnaFsdSView_finalize(GObject *object)
{
	GJnaFsdSView *self=(GJnaFsdSView*)object;

	gtk_widget_hide(GTK_WIDGET(self));
	g_object_unref(G_OBJECT(self->treeView));
	g_object_unref (self->model);
	gjnaFsdSView_parentClass->finalize(object);
}

static void
GJnaFsdSViewClass_init (GJnaFsdSViewClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	gjnaFsdSView_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_GJnaFsdSView_finalize;
}

GType		GJnaFsdSView_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (GJnaFsdSViewClass),
			NULL, NULL,
			(GClassInitFunc) GJnaFsdSViewClass_init,
			NULL, NULL,
			sizeof (GJnaFsdSView),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			GTK_TYPE_SCROLLED_WINDOW,
			"GJnaFsdSView", &info, 0);
	}

	return type;
}

static void
impl_GJnaFsdSView_selectionChanged 	(GtkTreeSelection *selection)
{
	GtkTreeView *treeview;
	GtkTreeModel *model;
	GtkTreeIter iter;
	GJnaFsdSView* self;

	treeview = gtk_tree_selection_get_tree_view (selection);

	self=g_object_get_data(G_OBJECT(treeview), "GJnaFsdSView");
	g_return_if_fail(GJNA_IS_FSD_S_VIEW(self));
	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
		JentchinaIter jIter; 

		jIter=GPOINTER_TO_INT(iter.user_data);
		if(self->dialog)
			GJnaFilterSelectionDialog_CB_selectionChanged(GJNA_FILTER_SELECTION_DIALOG(self->dialog), jIter);

	}
}

GObject* 	
GJnaFsdSView_new			(GObject* jentchina,
					 JentchinaFilterType filterType,
					 JentchinaTreeModelType modelType,
					 GObject* dialog)
{
	GJnaFsdSView* retval;
	GObject* jentchinaTreeModel;
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
	GtkTreeSelection *selection;

	g_return_val_if_fail(JENTCHINA_IS_OBJ(jentchina), NULL);
	
	retval=g_object_new(GJNA_TYPE_FSD_S_VIEW, NULL);
	retval->dialog=NULL;
	gtk_widget_set_usize(GTK_WIDGET(retval), 200, 400);
	if(dialog) {
		g_return_val_if_fail(GJNA_IS_FILTER_SELECTION_DIALOG(dialog), NULL);
		retval->dialog=dialog;
	}

	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (retval),
					   GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (retval),
				      GTK_POLICY_AUTOMATIC,
				      GTK_POLICY_AUTOMATIC);

	/* create gtk model from jentchina model */

	jentchinaTreeModel=JentchinaObj_getNewModel(JENTCHINA_OBJ(jentchina), filterType, modelType);
	retval->model=GJnaFsdModel_new(jentchinaTreeModel);
	g_object_unref(jentchinaTreeModel);

	/* create tree view */
	retval->treeView = gtk_tree_view_new_with_model (GTK_TREE_MODEL(retval->model));
	g_object_set_data(G_OBJECT(retval->treeView), "GJnaFsdSView", retval);
	gtk_widget_show(GTK_WIDGET(retval->treeView));
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (retval->treeView), TRUE); /* FIXME: what is this for ?*/
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW (retval->treeView), FALSE);
	selection=gtk_tree_view_get_selection (GTK_TREE_VIEW (retval->treeView));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

	g_signal_connect (selection, "changed", G_CALLBACK(impl_GJnaFsdSView_selectionChanged), NULL);
	gtk_container_add (GTK_CONTAINER (retval), retval->treeView);

	renderer = gtk_cell_renderer_text_new();
	col = gtk_tree_view_column_new();

	gtk_tree_view_column_pack_start (col, renderer, TRUE);
	gtk_tree_view_column_add_attribute (col, renderer, "text", 0);
	gtk_tree_view_column_set_title (col, "Name");
	gtk_tree_view_append_column(GTK_TREE_VIEW(retval->treeView),col);

	/* expand all rows after the treeview widget has been realized */
	g_signal_connect (retval->treeView, "realize",
			G_CALLBACK (gtk_tree_view_expand_all), NULL);
	

	return (GObject*) retval;
}

