/* GtkJentchina
 * STRASSER Michel - 20040408		init
 * */


#include "GJnaObj.h"
#include <glib.h>
#include <glib-object.h>
#include <libJentchina.h>
#include <lsmlib.h>
#include <IG.h>
#include "FilterSelectionDialog/pack_FilterSelectionDialog.h"

static GObjectClass *gjnaObj_parentClass;

static void
impl_GJnaObj_finalize(GObject *object)
{
	GJnaObj *self=(GJnaObj*)object;
	int i;

	gtk_widget_hide(GTK_WIDGET(self));
	g_object_unref(G_OBJECT(self->jentchina));
	for(i=0 ; i < GJnaFilterType_NUM  ; i++) 
		g_object_unref(self->fsd[i]);

	gjnaObj_parentClass->finalize(object);
}

static void
GJnaObjClass_init (GJnaObjClass *selfClass)
{
	GObjectClass *objectClass = (GObjectClass *)selfClass;
	gjnaObj_parentClass = g_type_class_peek_parent (selfClass);

	objectClass->finalize=impl_GJnaObj_finalize;
}

GType		GJnaObj_getType		(void)
{
	static GType type = 0;

	if (!type) {
		GTypeInfo info = {
			sizeof (GJnaObjClass),
			NULL, NULL,
			(GClassInitFunc) GJnaObjClass_init,
			NULL, NULL,
			sizeof (GJnaObj),
			0,
			NULL, NULL
		};

		type = g_type_register_static (
			G_TYPE_OBJECT,
			"GJnaObj", &info, 0);
	}

	return type;
}

GObject* 	
GJnaObj_new		(char* appName)
{
	GJnaObj* retval;
	int i;

	retval=g_object_new(GJNA_TYPE_OBJ, NULL);

	retval->jentchina=JentchinaObj_new(appName);

	for(i=0 ; i < GJnaFilterType_NUM  ; i++) {
		retval->fsd[i]=GJnaFilterSelectionDialog_new(appName, i, retval->jentchina);
		g_signal_connect ((gpointer) retval->fsd[i], "destroy_event", G_CALLBACK (gtk_widget_hide), NULL);
		g_signal_connect ((gpointer) retval->fsd[i], "delete_event", G_CALLBACK (gtk_widget_hide), NULL);
	}
	
	return (GObject*) retval;
}

static gboolean /* returns FALSE if cancelled */
impl_GJnaObj_interfaces	(GJnaObj* self,
			 GList* list,
			 char** out_filledInterfacesStream)
{
	gboolean retval;
	GList* l;
	IG* ig;

	ig=IG_new(FALSE);

	for(l=list ; l ; l=g_list_next(l)) {
		char *file=l->data;
		IG_addInterfaceFromFile(ig, file);
	}

	retval=!IG_run(ig);

	if(retval) {
		*out_filledInterfacesStream=IG_toXml(ig);
	}
	
	return retval;
}

GJnaStatus
GJnaObj_applyFilter	(GJnaObj* self,
			 GJnaFilterType filterType,
			 LsmTree* in_input,
			 LsmTree** out_output)
{
	GJnaStatus retval=GJnaStatus_OK;
	int r;
	JentchinaIter iter;
	gboolean bad=TRUE;
	char* sessionKey;
	GList* interfacesList;
	int vortexLifeTime;
	char* filledInterfaces;
	char filledInterfacesDefault[]=
		"<?xml version='1.0'?>"
		"<IGenStream local='no'>"
		"<!--Generated by LibIGen-->"
		"  <IGen name='/' label='/'>"
		"  </IGen>"
		"  <LsmTree hasContent='yes' VFS_defaultAuth='rwxrwxrwx' VFS_defaultOwner='root' VFS_defaultGroup='root'>"
		"    <LsmFiles/>"
		"  </LsmTree>"
		"</IGenStream>";
	gboolean notCancelled;
	JentchinaStatus status;

	while(bad) {
		r=gtk_dialog_run(GTK_DIALOG(self->fsd[filterType]));
		g_message("r is %d", r);
		if (r==GTK_RESPONSE_DELETE_EVENT || r==GTK_RESPONSE_CANCEL) {
			gtk_widget_hide(GTK_WIDGET(self->fsd[filterType]));
			return GJnaStatus_CANCELLED_BY_USER_ON_SELECTION;
		}
		g_return_val_if_fail(r==GTK_RESPONSE_OK, GJnaStatus_CANCELLED_BY_USER_ON_SELECTION); /* unexpected value ignored */

		/* TODO : test validity */
		if(1)
			bad=FALSE;
	}

	gtk_widget_hide(GTK_WIDGET(self->fsd[filterType]));
	iter=GJNA_FILTER_SELECTION_DIALOG(self->fsd[filterType])->filterRef;

	status=JentchinaObj_getRendezVous(JENTCHINA_OBJ(self->jentchina), iter, "michel", "secret", in_input, &interfacesList,
			&vortexLifeTime, &sessionKey);

	g_message("status : %d", status);

	g_return_val_if_fail(status==JentchinaStatus_OK, GJnaStatus_APPLY_FILTER_ERROR);
	g_message("sessionKey is '%s'", sessionKey);

	if(interfacesList)
		notCancelled=impl_GJnaObj_interfaces(self, interfacesList, &filledInterfaces);
	else {
		notCancelled=1;
		filledInterfaces=g_strdup(filledInterfacesDefault);
	}

	g_message("notCancelled='%d'", notCancelled);
	if(!notCancelled) 
		return GJnaStatus_CANCELLED_BY_USER_ON_OPTIONS;

	status=JentchinaObj_applyFilter(JENTCHINA_OBJ(self->jentchina), sessionKey, filledInterfaces, out_output);

	g_return_val_if_fail(status==JentchinaStatus_OK, GJnaStatus_APPLY_FILTER_ERROR);
		
	return retval;
}


